// Empfangen von IR-Fernsteuerung mit Fototransistor-Spannungsteiler an Pin 1.1
// 2010-12-28: von jl erstellt

#include "ir_receiver.h"
#include "io430.h"
#include "intrinsics.h"
#include <stdbool.h>
#include <stdio.h>

static volatile signed char numberOfPulses = -1;
static volatile unsigned char lagsBetweenPulses[maximumNumberOfLags];
static volatile unsigned int previousTimerValue;
static const int stopAfterNumberOfWdtInterrupts = 3;
static volatile unsigned char numberOfWdtInterrupts;
static volatile bool alreadySignalled = true;
static volatile unsigned char header;
static void (* volatile commandCallback)(volatile unsigned char number, volatile unsigned char lags[]);

#pragma vector = PORT1_VECTOR
__interrupt void P1_ISR(void)
{
  numberOfWdtInterrupts = 0;
  alreadySignalled = false;
  signed char n = numberOfPulses;
  n -= header;
  if(n < maximumNumberOfLags)
  {
    int currentTimerValue = TAR;
    if(n >= 0)
    {
      // Ignoriere berlauf usw., Hauptsache, wir haben eine charakteristische Zahl fr den Abstand,
      // auch wenn das dann keine echten Mikrosekunden sind.
      unsigned char difference = (unsigned char)(currentTimerValue - previousTimerValue);
      lagsBetweenPulses[n] = difference;
    }
    numberOfPulses++;
    previousTimerValue = currentTimerValue;
  }
  P1IFG &= ~BIT1;
}

#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR(void)
{
  if(!alreadySignalled)
  {
    numberOfWdtInterrupts++;
    if(numberOfWdtInterrupts > stopAfterNumberOfWdtInterrupts)
    {
      unsigned char n = (unsigned char)numberOfPulses;
      if(n < header)
      {
        n = 0;
      }
      else
      {
        n -= header;
      }
      commandCallback(n, lagsBetweenPulses);
      alreadySignalled = true;
      numberOfPulses = -1;
    }
  }
}

void initialize(unsigned char headerPulses, void (* volatile command)(volatile unsigned char number, volatile unsigned char lags[]))
{
  WDTCTL = WDTPW | WDTTMSEL;
  
  header = headerPulses;
  commandCallback = command;
  
  BCSCTL1 = CALBC1_1MHZ;
  DCOCTL = CALDCO_1MHZ;
  TACTL = MC_2 | TASSEL_2 | TACLR;
  
  P1DIR &= ~BIT1;
  P1REN &= ~BIT1;
  P1IES &= ~BIT1;
  P1IE |= BIT1;

  IE1 |= WDTIE;
  __enable_interrupt();
}

void write(unsigned int a)
{
  // hchste enthaltene Zehnerpotenz bestimmen
  // bzw. 1 bei Eingabe 0
  unsigned int p = 1;
  for(unsigned int b = a; b >= 10; b /= 10)
  {
    p *= 10;
  }

  // Ziffern startend bei der hchstwertigsten ausgeben
  for(; p > 0; p /= 10)
  {
    putchar(a / p + '0');
    a %= p;
  }

  putchar('\n');
}